@@ -21,15 +21,21 @@ library(kableExtra)
 
 ## ----brnrtab, message=FALSE, echo=FALSE---------------------------------------
 data.frame(Platform = c("Target 96",
-                         "Explore 384 Cardiometabolic, Inflammation, Neurology, and Oncology",
-                        "Explore 384 Cardiometabolic II, Inflammation II, Neurology II, and Oncology II",
-                        "Explore HT", 
-                        "Explore 3072 to Explore HT"),
+                        paste0("Explore 384: \n",
+                               "Cardiometabolic, Inflammation, ",
+                               "Neurology, and Oncology"),
+                        paste0("Explore 384: \n",
+                               "Cardiometabolic II, Inflammation II,",
+                               "Neurology II, and Oncology II"),
+                        "Explore HT",
+                        "Explore 3072 to Explore HT",
+                        "Explore 3072 to Reveal"),
            BridgingSamples = c("8-16",
                                "8-16",
-                              "16-24",
-                              "16-32",
-                              "40-64")) %>%
+                               "16-24",
+                               "16-32",
+                               "40-64",
+                               "32-48")) %>%
   kbl(booktabs = TRUE,
       digits = 2,
       caption = "Table 1. Recommended number of bridging samples for Olink platforms") %>%
